@@ -4,6 +4,7 @@
 
 #include "OgreApplicationContextBase.h"
 
+#include "OgreOverlayManager.h"
 #include "OgreRoot.h"
 #include "OgreGpuProgramManager.h"
 #include "OgreConfigFile.h"
@@ -20,6 +21,11 @@
 
 #include "OgreConfigPaths.h"
 
+#ifdef HAVE_IMGUI
+#include "OgreImGuiOverlay.h"
+#include "OgreImGuiInputListener.h"
+#endif
+
 namespace OgreBites {
 
 static const char* SHADER_CACHE_FILENAME = "cache.bin";
@@ -96,6 +102,9 @@ bool ApplicationContextBase::initialiseRTShaderSystem()
     {
         mShaderGenerator = Ogre::RTShader::ShaderGenerator::getSingletonPtr();
 
+        if(getRenderWindow()->isHardwareGammaEnabled())
+            mShaderGenerator->setTargetLinearColours(true);
+
         // Create and register the material manager listener if it doesn't exist yet.
         if (!mMaterialMgrListener) {
             mMaterialMgrListener = new SGTechniqueResolverListener(mShaderGenerator);
@@ -251,6 +260,77 @@ void ApplicationContextBase::destroyDummyScene()
     mRoot->destroySceneManager(dummyScene);
 }
 
+#ifdef HAVE_IMGUI
+struct ImGuiConfigDialog : Ogre::FrameListener
+{
+    Ogre::String nextRenderer;
+    bool saveConfig;
+
+    bool frameStarted(const Ogre::FrameEvent& evt) override
+    {
+        Ogre::ImGuiOverlay::NewFrame();
+
+        auto flags = ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse |
+                     ImGuiWindowFlags_NoSavedSettings | ImGuiWindowFlags_NoMove;
+        auto center = ImGui::GetMainViewport()->GetCenter();
+        ImGui::SetNextWindowPos(center, ImGuiCond_Always, ImVec2(0.5f, 0.5f));
+        ImGui::Begin("Rendering Settings", NULL, flags);
+        Ogre::DrawRenderingSettings(nextRenderer);
+        ImGui::Separator();
+        if (ImGui::Button("Accept"))
+        {
+            Ogre::Root::getSingleton().queueEndRendering();
+            saveConfig = true;
+        }
+        ImGui::SameLine();
+        if (ImGui::Button("Cancel"))
+            Ogre::Root::getSingleton().queueEndRendering();
+        ImGui::End();
+        return true;
+    }
+};
+#endif
+
+void ApplicationContextBase::runRenderingSettingsDialog()
+{
+#ifndef HAVE_IMGUI
+    OGRE_EXCEPT(Ogre::Exception::ERR_NOT_IMPLEMENTED,
+                "OGRE_BUILD_COMPONENT_OVERLAY_IMGUI=ON is required to run the rendering settings dialog");
+#else
+    createRoot();
+    oneTimeConfig();
+    auto rs = getRoot()->getRenderSystem();
+    auto oldVm = rs->getConfigOptions().at("Video Mode").currentValue;
+    rs->setConfigOption("Video Mode", "800 x 600");
+    setup();
+    rs->setConfigOption("Video Mode", oldVm);
+    createDummyScene();
+
+    float vpScale = getDisplayDPI()/96;
+    Ogre::OverlayManager::getSingleton().setPixelRatio(vpScale);
+    auto overlay = initialiseImGui();
+    ImGui::GetIO().FontGlobalScale = std::round(vpScale); // default font does not work with fractional scaling
+    overlay->show();
+
+    addInputListener(getImGuiInputListener());
+
+    ImGuiConfigDialog dialog;
+    getRoot()->addFrameListener(&dialog);
+    getRoot()->startRendering();
+
+    destroyDummyScene();
+    shutdown();
+    mRoot->shutdown();
+    if(dialog.saveConfig)
+    {
+        mRoot->setRenderSystem(mRoot->getRenderSystemByName(dialog.nextRenderer));
+        mRoot->saveConfig();
+    }
+    delete mRoot;
+    mRoot = NULL;
+#endif
+}
+
 void ApplicationContextBase::enableShaderCache() const
 {
     Ogre::GpuProgramManager::getSingleton().setSaveMicrocodesToCache(true);
@@ -405,6 +485,9 @@ void ApplicationContextBase::_fireInputEvent(const Event& event, uint32_t window
         case CONTROLLERAXISMOTION:
             l.axisMoved(event.axis);
             break;
+        case JOYHATMOTION:
+            l.hatMoved(event.hat);
+            break;
         case CONTROLLERBUTTONDOWN:
             l.buttonPressed(event.cbutton);
             break;
@@ -495,6 +578,29 @@ void ApplicationContextBase::loadResources()
     Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
 }
 
+Ogre::ImGuiOverlay* ApplicationContextBase::initialiseImGui()
+{
+#ifndef HAVE_IMGUI
+    OGRE_EXCEPT(Ogre::Exception::ERR_NOT_IMPLEMENTED,
+                "OGRE_BUILD_COMPONENT_OVERLAY_IMGUI=ON is required to use ImGui");
+    return NULL;
+#else
+        if(auto overlay = Ogre::OverlayManager::getSingleton().getByName("ImGuiOverlay"))
+            return static_cast<Ogre::ImGuiOverlay*>(overlay);
+
+        auto imguiOverlay = new Ogre::ImGuiOverlay();
+        Ogre::OverlayManager::getSingleton().addOverlay(imguiOverlay); // now owned by overlaymgr
+
+        // handle DPI scaling
+        float vpScale = Ogre::OverlayManager::getSingleton().getPixelRatio();
+        ImGui::GetStyle().ScaleAllSizes(vpScale);
+
+        mImGuiListener.reset(new ImGuiInputListener());
+
+        return imguiOverlay;
+#endif
+}
+
 void ApplicationContextBase::reconfigure(const Ogre::String &renderer, Ogre::NameValuePairList &options)
 {
     mNextRenderer = renderer;
