@@ -49,7 +49,7 @@ THE SOFTWARE.
 #include "OgreGLES2StateCacheManager.h"
 #include "OgreRenderWindow.h"
 #include "OgreGLES2PixelFormat.h"
-#include "OgreGLES2FBOMultiRenderTarget.h"
+#include "OgreGLMultiRenderTarget.h"
 
 #if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
 #include "OgreEAGLES2Context.h"
@@ -170,9 +170,9 @@ namespace Ogre {
 #if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID || OGRE_PLATFORM == OGRE_PLATFORM_EMSCRIPTEN
         mResourceManager = OGRE_NEW GLES2ManagedResourceManager();
 #endif
-        
+
         mGLSupport = getGLSupport(GLNativeSupport::CONTEXT_ES);
-        
+
 #if OGRE_PLATFORM != OGRE_PLATFORM_APPLE_IOS && OGRE_PLATFORM != OGRE_PLATFORM_ANDROID && OGRE_PLATFORM != OGRE_PLATFORM_WIN32
         glsupport = mGLSupport;
 #endif
@@ -299,7 +299,7 @@ namespace Ogre {
                checkExtension("GL_IMG_texture_compression_pvrtc2") ||
                checkExtension("WEBGL_compressed_texture_pvrtc"))
                 rsc->setCapability(RSC_TEXTURE_COMPRESSION_PVRTC);
-                
+
             if((checkExtension("GL_EXT_texture_compression_dxt1") &&
                checkExtension("GL_EXT_texture_compression_s3tc")) ||
                checkExtension("WEBGL_compressed_texture_s3tc"))
@@ -339,6 +339,8 @@ namespace Ogre {
             glGetIntegerv(GL_MAX_DRAW_BUFFERS, &buffers);
             rsc->setNumMultiRenderTargets(
                 std::min<int>(buffers, (GLint)OGRE_MAX_MULTIPLE_RENDER_TARGETS));
+
+            rsc->setCapability(RSC_RTT_DEPTHBUFFER_RESOLUTION_LESSEQUAL);
         }
         else
         {
@@ -347,7 +349,7 @@ namespace Ogre {
 
         // Stencil wrapping
         rsc->setCapability(RSC_STENCIL_WRAP);
-        
+
         // Point size
         GLfloat psRange[2] = {0.0, 0.0};
         OGRE_CHECK_GL_ERROR(glGetFloatv(GL_ALIASED_POINT_SIZE_RANGE, psRange));
@@ -355,8 +357,7 @@ namespace Ogre {
 
         // Point sprites
         rsc->setCapability(RSC_POINT_SPRITES);
-        rsc->setCapability(RSC_POINT_EXTENDED_PARAMETERS);
-        
+
         // GLSL ES is always supported in GL ES 2
         rsc->addShaderProfile("glsles");
         if (getNativeShadingLanguageVersion() >= 320)
@@ -424,7 +425,7 @@ namespace Ogre {
         // Alpha to coverage always 'supported' when MSAA is available
         // although card may ignore it if it doesn't specifically support A2C
         rsc->setCapability(RSC_ALPHA_TO_COVERAGE);
-        
+
         // No point sprites, so no size
         rsc->setMaxPointSize(0.f);
 
@@ -443,7 +444,10 @@ namespace Ogre {
         }
 
         if(hasMinGLVersion(3, 0))
+        {
             rsc->setCapability(RSC_VERTEX_FORMAT_INT_10_10_10_2);
+            rsc->setCapability(RSC_VERTEX_FORMAT_16X3);
+        }
 
         if (hasMinGLVersion(3, 0) || checkExtension("GL_EXT_instanced_arrays"))
         {
@@ -486,7 +490,7 @@ namespace Ogre {
         return rsc;
     }
 
-    void GLES2RenderSystem::initialiseFromRenderSystemCapabilities(RenderSystemCapabilities* caps, RenderTarget* primary)
+    void GLES2RenderSystem::initialiseFromRenderSystemCapabilities(RenderSystemCapabilities* caps, RenderTarget*)
     {
         if(caps->getNumVertexAttributes() < 16)
             GLSLProgramCommon::useTightAttributeLayout();
@@ -609,18 +613,6 @@ namespace Ogre {
                 mCurrentContext->setInitialized();
         }
 
-        if( win->getDepthBufferPool() != DepthBuffer::POOL_NO_DEPTH )
-        {
-            // Unlike D3D9, OGL doesn't allow sharing the main depth buffer, so keep them separate.
-            GLContext *windowContext = dynamic_cast<GLRenderTarget*>(win)->getContext();
-            auto depthBuffer =
-                new GLDepthBufferCommon(DepthBuffer::POOL_DEFAULT, this, windowContext, 0, 0, win, true);
-
-            mDepthBufferPool[depthBuffer->getPoolId()].push_back( depthBuffer );
-
-            win->attachDepthBuffer( depthBuffer );
-        }
-
         return win;
     }
 
@@ -648,7 +640,7 @@ namespace Ogre {
                                                        fbo->getHeight(), fbo->getFSAA() );
             }
 
-            return new GLDepthBufferCommon(0, this, mCurrentContext, depthBuffer, stencilBuffer,
+            return new GLDepthBufferCommon(this, mCurrentContext, depthBuffer, stencilBuffer,
                                            renderTarget, false);
         }
 
@@ -657,62 +649,11 @@ namespace Ogre {
 
     MultiRenderTarget* GLES2RenderSystem::createMultiRenderTarget(const String & name)
     {
-        MultiRenderTarget* retval =
-            new GLES2FBOMultiRenderTarget(static_cast<GLES2FBOManager*>(mRTTManager), name);
+        MultiRenderTarget* retval = new GLMultiRenderTarget(name, new GLES2FrameBufferObject());
         attachRenderTarget(*retval);
         return retval;
     }
 
-    void GLES2RenderSystem::destroyRenderWindow(const String& name)
-    {
-        // Find it to remove from list.
-        RenderTarget* pWin = detachRenderTarget(name);
-        OgreAssert(pWin, "unknown RenderWindow name");
-
-        _destroyDepthBuffer(pWin);
-        OGRE_DELETE pWin;
-    }
-
-    void GLES2RenderSystem::_destroyDepthBuffer(RenderTarget* pWin)
-    {
-        GLContext *windowContext = dynamic_cast<GLRenderTarget*>(pWin)->getContext();
-        
-        // 1 Window <-> 1 Context, should be always true
-        assert( windowContext );
-        
-        bool bFound = false;
-        // Find the depth buffer from this window and remove it.
-        DepthBufferMap::iterator itMap = mDepthBufferPool.begin();
-        DepthBufferMap::iterator enMap = mDepthBufferPool.end();
-        
-        while( itMap != enMap && !bFound )
-        {
-            DepthBufferVec::iterator itor = itMap->second.begin();
-            DepthBufferVec::iterator end  = itMap->second.end();
-            
-            while( itor != end )
-            {
-                // A DepthBuffer with no depth & stencil pointers is a dummy one,
-                // look for the one that matches the same GL context
-                auto depthBuffer = static_cast<GLDepthBufferCommon*>(*itor);
-                GLContext *glContext = depthBuffer->getGLContext();
-                
-                if( glContext == windowContext &&
-                   (depthBuffer->getDepthBuffer() || depthBuffer->getStencilBuffer()) )
-                {
-                    bFound = true;
-                    
-                    delete *itor;
-                    itMap->second.erase( itor );
-                    break;
-                }
-                ++itor;
-            }
-            
-            ++itMap;
-        }
-    }
-
     void GLES2RenderSystem::_setTexture(size_t stage, bool enabled, const TexturePtr &texPtr)
     {
         mStateCacheManager->activateGLTextureUnit(stage);
@@ -866,11 +807,11 @@ namespace Ogre {
         else if (vp != mActiveViewport || vp->_isUpdated())
         {
             RenderTarget* target;
-            
+
             target = vp->getTarget();
             _setRenderTarget(target);
             mActiveViewport = vp;
-            
+
             // Calculate the "lower-left" corner of the viewport
             Rect vpRect = vp->getActualDimensions();
             if (!target->requiresTextureFlipping())
@@ -891,8 +832,7 @@ namespace Ogre {
             // On iOS RenderWindow is FBO based, renders to multisampled FBO and then resolves
             // to non-multisampled FBO, therefore we need to restore FBO binding even when
             // rendering to the same viewport.
-            RenderTarget* target = vp->getTarget();
-            mRTTManager->bind(target);
+            bindRenderTarget(vp->getTarget());
         }
 #endif
     }
@@ -1008,7 +948,7 @@ namespace Ogre {
     {
         if(hasMinGLVersion(3, 0) || checkExtension("GL_EXT_occlusion_query_boolean"))
         {
-            GLES2HardwareOcclusionQuery* ret = new GLES2HardwareOcclusionQuery(); 
+            GLES2HardwareOcclusionQuery* ret = new GLES2HardwareOcclusionQuery();
             mHwOcclusionQueries.push_back(ret);
             return ret;
         }
@@ -1184,12 +1124,12 @@ namespace Ogre {
         else // VAOs are not supported, we should clear scratch 'VAO #0'
         {
             // Unbind all attributes
-            for(std::vector<GLuint>::iterator ai = mRenderAttribsBound.begin(); ai != mRenderAttribsBound.end(); ++ai)
-                OGRE_CHECK_GL_ERROR(glDisableVertexAttribArray(*ai));
+            for(const auto& b : mRenderAttribsBound)
+                OGRE_CHECK_GL_ERROR(glDisableVertexAttribArray(b));
 
             // Unbind any instance attributes
-            for (std::vector<GLuint>::iterator ai = mRenderInstanceAttribsBound.begin(); ai != mRenderInstanceAttribsBound.end(); ++ai)
-                OGRE_CHECK_GL_ERROR(glVertexAttribDivisorEXT(*ai, 0));
+            for(const auto& b : mRenderInstanceAttribsBound)
+                OGRE_CHECK_GL_ERROR(glVertexAttribDivisorEXT(b, 0));
         }
         mRenderAttribsBound.clear();
         mRenderInstanceAttribsBound.clear();
@@ -1299,7 +1239,7 @@ namespace Ogre {
             mProgramManager->setActiveShader(GPT_VERTEX_PROGRAM, NULL);
         if (mCurrentFragmentProgram)
             mProgramManager->setActiveShader(GPT_FRAGMENT_PROGRAM, NULL);
-        
+
         // Disable textures
         _disableTextureUnitsFrom(0);
 
@@ -1331,7 +1271,7 @@ namespace Ogre {
             mProgramManager->setActiveShader(GPT_VERTEX_PROGRAM, mCurrentVertexProgram);
         if (mCurrentFragmentProgram)
             mProgramManager->setActiveShader(GPT_FRAGMENT_PROGRAM, mCurrentFragmentProgram);
-        
+
         // Must reset depth/colour write mask to according with user desired, otherwise,
         // clearFrameBuffer would be wrong because the value we are recorded may be
         // difference with the really state stored in GL context.
@@ -1347,16 +1287,16 @@ namespace Ogre {
     {
         if(HardwareBufferManager::getSingletonPtr())
             static_cast<GLES2HardwareBufferManager*>(HardwareBufferManager::getSingletonPtr())->notifyContextDestroyed(context);
-        
-        for(RenderTargetMap::iterator it = mRenderTargets.begin(); it!=mRenderTargets.end(); ++it)
+
+        for(const auto& t : mRenderTargets)
         {
-            if(auto target = dynamic_cast<GLRenderTarget*>(it->second))
+            if(auto target = dynamic_cast<GLRenderTarget*>(t.second))
             {
                 if(auto fbo = target->getFBO())
                     fbo->notifyContextDestroyed(context);
             }
         }
-        
+
         if (mCurrentContext == context)
         {
             // Change the context to something else so that a valid context
@@ -1392,7 +1332,7 @@ namespace Ogre {
         else
             OGRE_CHECK_GL_ERROR(glDeleteVertexArraysOES(1, &vao));
     }
-    
+
     void GLES2RenderSystem::_destroyFbo(GLContext* context, uint32 fbo)
     {
         if(context != mCurrentContext)
@@ -1459,6 +1399,24 @@ namespace Ogre {
         LogManager::getSingleton().logMessage("**************************************");
     }
 
+    void GLES2RenderSystem::bindRenderTarget(RenderTarget* target)
+    {
+        if(auto fbo = dynamic_cast<GLRenderTarget*>(target)->getFBO())
+        {
+            fbo->bind(true);
+        }
+        else
+        {
+            // Non-multisampled screen buffer is FBO #1 on iOS, multisampled is yet another,
+            // so give the target ability to influence decision which FBO to use
+            GLuint mainfbo = 0;
+#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
+            target->getCustomAttribute("GLFBO", &mainfbo);
+#endif
+            OGRE_CHECK_GL_ERROR(glBindFramebuffer(GL_FRAMEBUFFER, mainfbo));
+        }
+    }
+
     void GLES2RenderSystem::_setRenderTarget(RenderTarget *target)
     {
         mActiveRenderTarget = target;
@@ -1474,7 +1432,7 @@ namespace Ogre {
             // Check the FBO's depth buffer status
             auto depthBuffer = static_cast<GLDepthBufferCommon*>(target->getDepthBuffer());
 
-            if( target->getDepthBufferPool() != DepthBuffer::POOL_NO_DEPTH &&
+            if( target->getDepthBufferPool() != RBP_NONE &&
                 (!depthBuffer || depthBuffer->getGLContext() != mCurrentContext ) )
             {
                 // Depth is automatically managed and there is no depth buffer attached to this RT
@@ -1483,7 +1441,7 @@ namespace Ogre {
             }
 
             // Bind frame buffer object
-            mRTTManager->bind(target);
+            bindRenderTarget(target);
         }
     }
 
@@ -1542,11 +1500,11 @@ namespace Ogre {
     {
         if (!prg)
         {
-            OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
+            OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
                         "Null program bound.",
                         "GLES2RenderSystem::bindGpuProgram");
         }
-        
+
         GLSLESProgram* glprg = static_cast<GLSLESProgram*>(prg);
 
         switch (glprg->getType())
@@ -1554,14 +1512,14 @@ namespace Ogre {
             case GPT_VERTEX_PROGRAM:
                 mCurrentVertexProgram = glprg;
                 break;
-                
+
             case GPT_FRAGMENT_PROGRAM:
                 mCurrentFragmentProgram = glprg;
                 break;
             default:
                 break;
         }
-        
+
         // Bind the program
         mProgramManager->setActiveShader(glprg->getType(), glprg);
 
@@ -1641,26 +1599,17 @@ namespace Ogre {
         LogManager::getSingleton().logMessage("********************************************");
         LogManager::getSingleton().logMessage("*** OpenGL ES 2.x Reset Renderer Started ***");
         LogManager::getSingleton().logMessage("********************************************");
-                
+
         initialiseContext(win);
-        
+
         static_cast<GLES2FBOManager*>(mRTTManager)->_reload();
-        
-        _destroyDepthBuffer(win);
-
-        auto depthBuffer =
-            new GLDepthBufferCommon(DepthBuffer::POOL_DEFAULT, this, mMainContext, 0, 0, win, true);
-
-        mDepthBufferPool[depthBuffer->getPoolId()].push_back( depthBuffer );
-        win->attachDepthBuffer( depthBuffer );
-        
         GLES2RenderSystem::mResourceManager->notifyOnContextReset();
-        
+
         mStateCacheManager->clearCache();
         _setViewport(NULL);
         _setRenderTarget(win);
     }
-    
+
     GLES2ManagedResourceManager* GLES2RenderSystem::getResourceManager()
     {
         return GLES2RenderSystem::mResourceManager;
@@ -1704,6 +1653,7 @@ namespace Ogre {
             case VET_USHORT2_NORM:
             case VET_SHORT4_NORM:
             case VET_USHORT4_NORM:
+            case VET_INT_10_10_10_2_NORM:
                 normalised = GL_TRUE;
                 break;
             default:
