@@ -37,7 +37,6 @@ THE SOFTWARE.
 #include "OgrePixelFormat.h"
 #include "OgreVulkanTextureGpuManager.h"
 #include "OgreHardwarePixelBuffer.h"
-#include "OgreViewport.h"
 #if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
 #include <android/native_window.h>
 #elif OGRE_PLATFORM == OGRE_PLATFORM_WIN32
@@ -52,7 +51,6 @@ namespace Ogre
 {
     VulkanWindow::VulkanWindow( const String &title, uint32 width, uint32 height, bool fullscreenMode ) :
         mLowestLatencyVSync( false ),
-        mHwGamma( false ),
         mDevice( 0 ),
         mTexture( 0 ),
         mDepthTexture( 0 ),
@@ -113,6 +111,14 @@ namespace Ogre
     //-------------------------------------------------------------------------
     void VulkanWindow::createSwapchain( void )
     {
+        VkSurfaceCapabilitiesKHR surfaceCaps;
+        OGRE_VK_CHECK(vkGetPhysicalDeviceSurfaceCapabilitiesKHR(mDevice->mPhysicalDevice, mSurfaceKHR, &surfaceCaps));
+
+        // Swapchain may be smaller/bigger than requested
+        mWidth = Math::Clamp(mWidth, surfaceCaps.minImageExtent.width, surfaceCaps.maxImageExtent.width);
+        mHeight =
+            Math::Clamp(mHeight, surfaceCaps.minImageExtent.height, surfaceCaps.maxImageExtent.height);
+
         mTexture->setWidth(mWidth);
         mTexture->setHeight(mHeight);
         mTexture->createInternalResources();
@@ -122,14 +128,6 @@ namespace Ogre
         mDepthTexture->setNumMipmaps(0);
         mDepthTexture->createInternalResources();
 
-        VkSurfaceCapabilitiesKHR surfaceCaps;
-        OGRE_VK_CHECK(vkGetPhysicalDeviceSurfaceCapabilitiesKHR(mDevice->mPhysicalDevice, mSurfaceKHR, &surfaceCaps));
-
-        // Swapchain may be smaller/bigger than requested
-        mWidth = Math::Clamp(mWidth, surfaceCaps.minImageExtent.width, surfaceCaps.maxImageExtent.width);
-        mHeight =
-            Math::Clamp(mHeight, surfaceCaps.minImageExtent.height, surfaceCaps.maxImageExtent.height);
-
         VkBool32 supported;
         OGRE_VK_CHECK(vkGetPhysicalDeviceSurfaceSupportKHR(mDevice->mPhysicalDevice, mDevice->mGraphicsQueue.mFamilyIdx,
                                                            mSurfaceKHR, &supported));
@@ -292,7 +290,16 @@ namespace Ogre
         uint32 imageIdx = 0u;
         auto res =
             vkAcquireNextImageKHR(mDevice->mDevice, mSwapchain, UINT64_MAX, semaphore, VK_NULL_HANDLE, &imageIdx);
-        if (res != VK_ERROR_OUT_OF_DATE_KHR && res != VK_SUBOPTIMAL_KHR && res != VK_SUCCESS)
+
+        if (res == VK_ERROR_OUT_OF_DATE_KHR || res == VK_SUBOPTIMAL_KHR)
+        {
+            mDevice->stall();
+            destroySwapchain();
+            createSwapchain();
+            return;
+        }
+
+        if (res != VK_SUCCESS)
         {
             LogManager::getSingleton().logError("vkAcquireNextImageKHR failed with" + vkResultToString(res));
             return;
@@ -310,21 +317,9 @@ namespace Ogre
         if (mClosed)
             return;
 
-        if (mWidth == width && mHeight == height)
-            return;
-
         if (width != 0 && height != 0)
         {
-            mWidth = width;
-            mHeight = height;
-
-            // recreate swapchain
-            mDevice->stall();
-            destroySwapchain();
-            createSwapchain();
-
-            for (auto it : mViewportList)
-                it.second->_updateDimensions();
+            RenderWindow::resize(width, height);
         }
     }
 
@@ -364,7 +359,7 @@ namespace Ogre
     //-------------------------------------------------------------------------
     void VulkanWindow::createSurface(size_t windowHandle)
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_LINUX
+#if defined(VK_USE_PLATFORM_XLIB_KHR)
         Display *dpy = XOpenDisplay(NULL);
 
         VkXlibSurfaceCreateInfoKHR surfCreateInfo = {VK_STRUCTURE_TYPE_XLIB_SURFACE_CREATE_INFO_KHR};
@@ -379,7 +374,7 @@ namespace Ogre
         }
 
         OGRE_VK_CHECK(vkCreateXlibSurfaceKHR(mDevice->mInstance, &surfCreateInfo, 0, &mSurfaceKHR));
-#elif OGRE_PLATFORM == OGRE_PLATFORM_WIN32
+#elif defined(VK_USE_PLATFORM_WIN32_KHR)
         HINSTANCE hInst = NULL;
         static TCHAR staticVar;
         GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, &staticVar, &hInst);
@@ -389,23 +384,41 @@ namespace Ogre
         surfCreateInfo.hwnd = (HWND)windowHandle;
 
         OGRE_VK_CHECK(vkCreateWin32SurfaceKHR(mDevice->mInstance, &surfCreateInfo, 0, &mSurfaceKHR));
-#elif OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
+#elif defined(VK_USE_PLATFORM_ANDROID_KHR)
         VkAndroidSurfaceCreateInfoKHR surfCreateInfo = {VK_STRUCTURE_TYPE_ANDROID_SURFACE_CREATE_INFO_KHR};
         surfCreateInfo.window = (ANativeWindow*)windowHandle;
 
         OGRE_VK_CHECK(vkCreateAndroidSurfaceKHR(mDevice->mInstance, &surfCreateInfo, 0, &mSurfaceKHR));
+#else
+        OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Unsupported Vulkan platform");
+#endif
+    }
+
+    void VulkanWindow::createSurface(size_t wlSurface, size_t wlDisplay)
+    {
+#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
+        VkWaylandSurfaceCreateInfoKHR surfCreateInfo = {VK_STRUCTURE_TYPE_WAYLAND_SURFACE_CREATE_INFO_KHR};
+        surfCreateInfo.display = (struct wl_display*)wlDisplay;
+        surfCreateInfo.surface = (struct wl_surface*)wlSurface;
+
+        OGRE_VK_CHECK(vkCreateWaylandSurfaceKHR(mDevice->mInstance, &surfCreateInfo, 0, &mSurfaceKHR));
+#else
+        OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Unsupported Vulkan platform");
 #endif
     }
 
     const char *VulkanWindow::getRequiredExtensionName()
     {
-#if OGRE_PLATFORM == OGRE_PLATFORM_LINUX
+#if defined(VK_USE_PLATFORM_XLIB_KHR)
         return VK_KHR_XLIB_SURFACE_EXTENSION_NAME;
-#elif OGRE_PLATFORM == OGRE_PLATFORM_WIN32
+#elif defined(VK_USE_PLATFORM_WAYLAND_KHR)
+        return VK_KHR_WAYLAND_SURFACE_EXTENSION_NAME;
+#elif defined(VK_USE_PLATFORM_WIN32_KHR)
         return VK_KHR_WIN32_SURFACE_EXTENSION_NAME;
-#elif OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
+#elif defined(VK_USE_PLATFORM_ANDROID_KHR)
         return VK_KHR_ANDROID_SURFACE_EXTENSION_NAME;
 #endif
+        return "";
     }
 
     void VulkanWindow::create(const String& name, unsigned int width, unsigned int height, bool fullScreen,
@@ -419,6 +432,8 @@ namespace Ogre
         mHeight = height;
         mFSAA = 1;
 
+        size_t wlDisplay = 0;
+
         if( miscParams )
         {
             NameValuePairList::const_iterator end = miscParams->end();
@@ -427,6 +442,15 @@ namespace Ogre
             if( opt != end )
                 mWindowHandle = StringConverter::parseSizeT( opt->second );
 
+            opt = miscParams->find( "externalWlSurface" );
+            if( opt != end )
+                mWindowHandle = StringConverter::parseSizeT( opt->second );
+
+
+            opt = miscParams->find( "externalWlDisplay" );
+            if( opt != end )
+                wlDisplay = StringConverter::parseSizeT( opt->second );
+
             opt = miscParams->find( "vsync" );
             if( opt != end )
                 mVSync = StringConverter::parseBool( opt->second );
@@ -441,8 +465,14 @@ namespace Ogre
                 mHwGamma = StringConverter::parseBool( opt->second );
         }
 
-        OgreAssert( mWindowHandle,  "externalWindowHandle required" );
+#ifdef VK_USE_PLATFORM_WAYLAND_KHR
+        OgreAssert( mWindowHandle && wlDisplay, "externalWlSurface and externalWlDisplay required");
+        createSurface(mWindowHandle, wlDisplay);
+#else
+        OgreAssert( mWindowHandle, "externalWindowHandle required");
         createSurface(mWindowHandle);
+        (void)wlDisplay;
+#endif
 
         auto texMgr = TextureManager::getSingletonPtr();
         mTexture = new VulkanTextureGpuWindow("RenderWindow", TEX_TYPE_2D, texMgr, this);;
